"""
Copyright (c) 2023 CEA-List

This program and the accompanying materials are made available under the
terms of the Eclipse Public License 2.0 which is available at
http://www.eclipse.org/legal/epl-2.0.

SPDX-License-Identifier: EPL-2.0
"""
from typing import List, Tuple

import aidge_core
import onnx

from aidge_onnx.node_import import auto_register_import

from aidge_core import Log
@auto_register_import("relu")
def import_relu(onnx_node:onnx.NodeProto, input_nodes:List[Tuple[aidge_core.Node, int]], opset: int = None) -> aidge_core.Node:
    """
    :param onnx_node: ONNX node to convert
    :type onnx_node: onnx.NodeProto
    :param input_nodes: List of pairs of Aidge Nodes which constitute the input
        of the current node and their associated output index linking to the
        current Node.
    :type input_nodes: List[aidge_core.Node, int]
    :param opset: Indicate opset version of the ONNX model, default=None
    :type opset: int, optional
    """
    node_name = onnx_node.name if onnx_node.name else onnx_node.output[0]
    onnx_attrs = {attr.name : attr for attr in onnx_node.attribute}

    if opset < 6:
        #### Attributes
        #  consumed_inputs (Optional) : list of ints
        #    legacy optimization attribute.
        #### Inputs
        #  X : T
        #    Input tensor
        #### Outputs
        #  Y : T
        #    Output tensor
        if 'consumed_inputs' in onnx_attrs:
            #legacy optimisation attribute, ignored
            del onnx_attrs['consumed_inputs']

    if len(onnx_attrs) > 0:
        Log.warn(f"Warning: unsupported attribute(s): {onnx_attrs.keys()} for operator 'Relu' with opset {opset}.\nThis node will be filled by a GenericOperator.")
        return None

    my_node = aidge_core.Node(aidge_core.ReLUOp(), name=node_name)
    Log.notice(f"- {node_name} ({onnx_node.op_type})")
    return my_node
